<?php
/**
* 2018 Tuxpan
* Clase cliente del Api2 de Flow
*
*  @author flow.cl
*  @copyright  2018 Tuxpan
*  @version: 2.0
*  @Email: soporte@tuxpan.com
*  @Date: 15-05-2018 11:00
*  @Last Modified by: Tuxpan
*  @Last Modified time: 15-05-2018 11:00
*/

include_once(dirname(__FILE__).'/../../../config/config.inc.php');

class FlowApiWP
{

    /**
     * Funcion que invoca un servicio del Api de Flow
     * @param string $service Nombre del servicio a ser invocado
     * @param array $params datos a ser enviados
     * @param string $method metodo http a utilizar
     * @return string en formato JSON
     */
    public function send($service, $params, $method = "GET")
    {
        $method = Tools::strtoupper($method);
        $flow_platform_id = Configuration::get('FLOW_WP_PLATFORM');
        $isTestPlatform = !$flow_platform_id || $flow_platform_id == 'test';
        $url_api = $isTestPlatform ? "https://sandbox.flow.cl/api" : "https://www.flow.cl/api";
        $url = $url_api . "/" . $service;
        $params = array("apiKey" => Configuration::get('FLOW_WP_APIKEY')) + $params;
        $toSign = $this->getPack($params, $method);
        if (!function_exists("hash_hmac")) {
            throw new Exception("function hash_hmac does not exist", 1);
        }
        $sign = hash_hmac('sha256', $toSign, Configuration::get('FLOW_WP_PRIVATEKEY'));

        if ($method == "GET") {
            $response = $this->httpGet($url, $toSign, $sign);
        } else {
            $response = $this->httpPost($url, $toSign, $sign);
        }
        if (empty($response["output"]) && $response["info"]["http_code"] != 200) {
            throw new Exception(
                "Unexpected error occurred. HTTP_CODE: " .$response["info"]["http_code"],
                $response["info"]["http_code"]
            );
        }
        $body = json_decode($response["output"], true);
        if ($response["info"]["http_code"] != 200) {
            throw new Exception($body["message"], $body["code"]);
        }
        return $body;
    }

    /**
     * Funcion que empaqueta los datos para ser firmados
     * @param array $params datos a ser empaquetados
     * @param string $method metodo http a utilizar
     */
    private function getPack($params, $method)
    {
        $keys = array_keys($params);
        sort($keys);
        $toSign = "";
        foreach ($keys as $key) {
            if ($method == "GET") {
                $toSign .= "&" . rawurlencode($key) . "=" . rawurlencode($params[$key]);
            } else {
                $toSign .= "&" . $key . "=" . $params[$key];
            }
        }
        return Tools::substr($toSign, 1);
    }

    /**
     * Funcion que hace el llamado via http GET
     * @param string $url url a invocar
     * @param array $data datos a enviar
     * @param string $sign firma de los datos
     * @return string en formato JSON
     */
    private function httpGet($url, $data, $sign)
    {
        $url = $url . "?" . $data . "&s=" . $sign;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $output = curl_exec($ch);
        if ($output === false) {
            $error = curl_error($ch);
            throw new Exception($error, 1);
        }
        $info = curl_getinfo($ch);
        curl_close($ch);
        return array("output" =>$output, "info" => $info);
    }

    /**
     * Funcion que hace el llamado via http POST
     * @param string $url url a invocar
     * @param array $data datos a enviar
     * @param string $sign firma de los datos
     * @return string en formato JSON
     */
    private function httpPost($url, $data, $sign)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data . "&s=" . $sign);
        $output = curl_exec($ch);
        if ($output === false) {
            $error = curl_error($ch);
            throw new Exception($error, 1);
        }
        $info = curl_getinfo($ch);
        curl_close($ch);
        return array("output" =>$output, "info" => $info);
    }

    private function customHmac($algo, $data, $key, $raw_output = false)
    {
        $algo = Tools::strtolower($algo);
        $pack = 'H'.Tools::strlen(hash($algo, 'test'));
        $size = 64;
        $opad = str_repeat(chr(0x5C), $size);
        $ipad = str_repeat(chr(0x36), $size);

        if (Tools::strlen($key) > $size) {
            $key = str_pad(pack($pack, hash($algo, $key)), $size, chr(0x00));
        } else {
            $key = str_pad($key, $size, chr(0x00));
        }

        for ($i = 0; $i < Tools::strlen($key) - 1; $i++) {
            $opad[$i] = $opad[$i] ^ $key[$i];
            $ipad[$i] = $ipad[$i] ^ $key[$i];
        }

        $output = hash($algo, $opad.pack($pack, hash($algo, $ipad.$data)));

        return ($raw_output) ? pack($pack, $output) : $output;
    }
}
